use std::{fs::File, io::BufReader};

#[derive(serde::Deserialize, Debug)]
pub(crate) struct Config {
    pub(crate) matrix: MatrixConfig,
    pub(crate) meshtastic: MeshtasticConfig,
}

#[derive(serde::Deserialize, Debug)]
pub(crate) struct MatrixConfig {
    pub(crate) username: String,
    pub(crate) password: String,
    #[serde(default = "get_device_name")]
    pub(crate) device_name: String,
    pub(crate) room: String,
}

#[derive(serde::Deserialize, Debug)]
pub(crate) struct MeshtasticConfig {
    pub(crate) device: String,
}

fn get_device_name() -> String {
    "meshtastic-bridge".to_string()
}

pub(crate) async fn read_config() -> Config {
    let file = File::open("config.json").expect("failed to read config.json");
    let reader = BufReader::new(file);

    // Read the JSON contents of the file as an instance of `User`.
    serde_json::from_reader(reader).expect("failed to parse config.json")
}
